
#include <string.h>
#include "light_ws2812.h"
#include <avr/io.h>

#define LED_NUM_WIDTH 39u
#define LED_NUM_HEIGTH 12u
#define ADC_MEAN_SAMPLES 32u

struct cRGB ledState[102];
const struct cRGB ledStateDefault[102] =
{
    {4u, 4u, 255u},
    {6u, 6u, 255u},
    {8u, 8u, 255u},
    {11u, 11u, 255u},
    {14u, 14u, 250u},
    {17u, 17u, 240u},
    {20u, 20u, 230u},
    {25u, 25u, 220u},
    {30u, 30u, 210u},
    {35u, 35u, 200u},
    {40u, 40u, 190u},
    {45u, 35u, 180u},
    {50u, 50u, 170u},
    {55u, 55u, 160u},
    {60u, 60u, 150u},
    {65u, 65u, 140u},
    {70u, 70u, 130u},
    {80u, 80u, 120u},
    {90u, 90u, 110u},
    {100u, 100u, 100u},
    {90u, 90u, 110u},
    {80u, 80u, 120u},
    {70u, 70u, 130u},
    {65u, 65u, 140u},
    {60u, 60u, 150u},
    {55u, 55u, 160u},
    {50u, 50u, 170u},
    {45u, 45u, 180u},
    {40u, 40u, 190u},
    {35u, 35u, 200u},
    {30u, 30u, 210u},
    {25u, 25u, 220u},
    {20u, 20u, 230u},
    {17u, 17u, 240u},
    {14u, 14u, 250u},
    {11u, 11u, 255u},
    {8u, 8u, 255u},
    {6u, 6u, 255u},
    {4u, 4u, 255u},

    {4u, 4u, 255u},
    {4u, 8u, 200},
    {4u, 15u, 150u},
    {4u, 20u, 100u},
    {4u, 60u, 70u},
    {4u, 100u, 50u},
    {4u, 100u, 50u},
    {4u, 60u, 70u},
    {4u, 20u, 100u},
    {4u, 15u, 150u},
    {4u, 8u, 200u},
    {4u, 4u, 255u},

    {4u, 4u, 255u},
    {6u, 6u, 255u},
    {8u, 8u, 255u},
    {11u, 11u, 255u},
    {14u, 14u, 250u},
    {17u, 17u, 240u},
    {20u, 20u, 230u},
    {25u, 25u, 220u},
    {30u, 30u, 210u},
    {35u, 35u, 200u},
    {40u, 40u, 190u},
    {45u, 35u, 180u},
    {50u, 50u, 170u},
    {55u, 55u, 160u},
    {60u, 60u, 150u},
    {65u, 65u, 140u},
    {70u, 70u, 130u},
    {80u, 80u, 120u},
    {90u, 90u, 110u},
    {100u, 100u, 100u},
    {90u, 90u, 110u},
    {80u, 80u, 120u},
    {70u, 70u, 130u},
    {65u, 65u, 140u},
    {60u, 60u, 150u},
    {55u, 55u, 160u},
    {50u, 50u, 170u},
    {45u, 45u, 180u},
    {40u, 40u, 190u},
    {35u, 35u, 200u},
    {30u, 30u, 210u},
    {25u, 25u, 220u},
    {20u, 20u, 230u},
    {17u, 17u, 240u},
    {14u, 14u, 250u},
    {11u, 11u, 255u},
    {8u, 8u, 255u},
    {6u, 6u, 255u},
    {4u, 4u, 255u},

    {4u, 4u, 255u},
    {4u, 8u, 200},
    {4u, 15u, 150u},
    {4u, 20u, 100u},
    {4u, 60u, 70u},
    {4u, 100u, 50u},
    {4u, 100u, 50u},
    {4u, 60u, 70u},
    {4u, 20u, 100u},
    {4u, 15u, 150u},
    {4u, 8u, 200u},
    {4u, 4u, 255u},
};

const int8_t currentSteps[LED_NUM_HEIGTH] = {
    7,
    10,
    15,
    25,
    35,
    40,
    48,
    55,
    65,
    80,
    100,
    120,
};

int main(void)
{
    int32_t adc_result_ch1 = 0u;
    int32_t adc_result_ch2 = 0u;
    uint8_t led_num;
    DDRA = (uint8_t)0x00u;
    ADCSRA = (uint8_t)0b10000111u;

    while(1)
    {
        adc_result_ch1 = 0;
        adc_result_ch2 = 0;
        for(uint32_t i = 0u; i < ADC_MEAN_SAMPLES; i ++)
        {
            // reading ch1
            ADCSRA |= (1<<ADSC);
            ADMUX = (uint8_t)0b010001001;
            while((ADCSRA&(1<<ADIF))==0);
            // ignore negative values
            if((ADCH & 0x02u) == 0u)
            {
                adc_result_ch1 += ((int16_t)ADCL + ((int16_t)ADCH << 8u)) >> 2u;
            }
            // reading ch2
            ADCSRA |= (1<<ADSC);
            ADMUX = (uint8_t)0b010001101;
            while((ADCSRA&(1<<ADIF))==0);
            if((ADCH & 0x02u) == 0u)
            {
                adc_result_ch2 += ((int16_t)ADCL + ((int16_t)ADCH << 8u)) >> 2u;
            }
        }
        adc_result_ch1 = adc_result_ch1 / ADC_MEAN_SAMPLES;
        adc_result_ch2 = adc_result_ch2 / ADC_MEAN_SAMPLES;

        // copy the default
        (void)memcpy(ledState, ledStateDefault, sizeof(ledState));

        // ch1 left
        for(uint32_t i = 0u; i < LED_NUM_HEIGTH; i ++)
        {
            led_num = LED_NUM_HEIGTH + LED_NUM_WIDTH - 1 - i;

            if (adc_result_ch1 >= currentSteps[i])
            {
                ledState[led_num].g = 0;
                ledState[led_num].b = 0;
                ledState[led_num].r = 255;
            }
            else if (i > 0 && adc_result_ch1 >= currentSteps[i - 1])
            {
                ledState[led_num].g = 0;
                ledState[led_num].b = 0;
                ledState[led_num].r = adc_result_ch1 - currentSteps[i - 1];
            }
        }

        // ch2 right
        for(uint32_t i = 0u; i < LED_NUM_HEIGTH; i ++)
        {
            led_num = LED_NUM_HEIGTH + (2 * LED_NUM_WIDTH) + i;

            if (adc_result_ch2 >= currentSteps[i])
            {
                ledState[led_num].g = 0;
                ledState[led_num].b = 0;
                ledState[led_num].r = 255;
            }
            else if (i > 0 && adc_result_ch2 >= currentSteps[i - 1])
            {
                ledState[led_num].g = 0;
                ledState[led_num].b = 0;
                ledState[led_num].r = adc_result_ch2 - currentSteps[i - 1];
            }
        }

        ws2812_setleds(ledState, 102);
    }
    return 0;
}